@@ -2,37 +2,40 @@
 Interact with Synapse discussion API endpoints.
 '''
 import json
-import requests
+from typing import Iterator, List, Union
 
+import requests
 import synapseclient
+from synapseclient import Project, Synapse, UserProfile
 from synapseclient.core.utils import id_of
 
-from .synapseservices.discussion import Forum
+from .synapseservices.discussion import Forum, Thread, Reply
 
 QUERY_LIMIT = 1000
 
 
 class DiscussionApi:
     """Discussion API calls"""
-    def __init__(self, syn=None):
+    def __init__(self, syn: Synapse = None):
         if syn is None:
             syn = synapseclient.login()
         self.syn = syn
 
-    def get_project_forum(self, projectid):
+    def get_project_forum(self, projectid: str) -> Forum:
         """Get the Forum's metadata for a given project ID.
         https://rest-docs.synapse.org/rest/GET/project/projectId/forum.html
         """
         return Forum(**self.syn.restGET(f'/project/{projectid}/forum'))
 
-    def get_forum(self, forumid):
+    def get_forum(self, forumid: str) -> Forum:
         """Get the Forum's metadata for a given forum ID.
         https://rest-docs.synapse.org/rest/GET/forum/forumId.html
         """
         return Forum(**self.syn.restGET(f'/forum/{forumid}'))
 
-    def get_forum_threads(self, forumid, query_filter='EXCLUDE_DELETED',
-                          limit=20, offset=0):
+    def get_forum_threads(self, forumid: str,
+                          query_filter: str = 'EXCLUDE_DELETED',
+                          **kwargs) -> Iterator[Thread]:
         """Get N number of threads for a given forum ID
         https://rest-docs.synapse.org/rest/GET/forum/forumId/threads.html
 
@@ -45,11 +48,14 @@ class DiscussionApi:
 
         Yields:
             list: Forum threads
+
         """
         uri = f'/forum/{forumid}/threads?filter={query_filter}'
-        return self.syn._GET_paginated(uri, limit=limit, offset=offset)
+        threads = self.syn._GET_paginated(uri, **kwargs)
+        for thread in threads:
+            yield Thread(**thread)
 
-    def post_thread(self, forumid, title, message):
+    def post_thread(self, forumid: str, title: str, message: str) -> Thread:
         """Create a new thread in a forum
         https://rest-docs.synapse.org/rest/POST/thread.html
 
@@ -64,10 +70,12 @@ class DiscussionApi:
         request_obj = {'forumId': forumid,
                        'title': title,
                        'messageMarkdown': message}
-        return self.syn.restPOST('/thread',
-                                 body=json.dumps(request_obj))
+        thread = self.syn.restPOST('/thread',
+                                   body=json.dumps(request_obj))
+        return Thread(**thread)
 
-    def get_threads_referencing_entity(self, entityid, limit=20, offset=0):
+    def get_threads_referencing_entity(self, entityid: str,
+                                       **kwargs) -> Iterator[Thread]:
         """
         Get N number of threads that belongs to projects user can
         view and references the given entity
@@ -80,59 +88,61 @@ class DiscussionApi:
         Yields:
             DiscussionThreadBundles
         """
-        return self.syn._GET_paginated(f"/entity/{entityid}/threads",
-                                       limit=limit, offset=offset)
+        threads = self.syn._GET_paginated(f"/entity/{entityid}/threads",
+                                          **kwargs)
+        for thread in threads:
+            yield Thread(**thread)
 
-    def get_thread(self, threadid):
+    def get_thread(self, threadid: str) -> Thread:
         """Get a thread and its statistic given its ID
         https://rest-docs.synapse.org/rest/GET/thread/threadId.html
         """
-        return self.syn.restGET(f"/thread/{threadid}")
+        return Thread(**self.syn.restGET(f"/thread/{threadid}"))
 
-    def update_thread_title(self, threadid):
+    def update_thread_title(self, threadid: str) -> Thread:
         """Update title of a thread
         https://rest-docs.synapse.org/rest/PUT/thread/threadId/title.html
         """
-        return self.syn.restPOST(f"/thread/{threadid}/title")
+        return Thread(**self.syn.restPUT(f"/thread/{threadid}/title"))
 
-    def update_thread_message(self, threadid):
+    def update_thread_message(self, threadid: str) -> Thread:
         """Update message of a thread
         https://rest-docs.synapse.org/rest/PUT/thread/threadId/message.html
         """
-        return self.syn.restPOST(f"/thread/{threadid}/message")
+        return Thread(**self.syn.restPUT(f"/thread/{threadid}/message"))
 
-    def delete_thread(self, threadid):
+    def delete_thread(self, threadid: str):
         """Delete thread
         https://rest-docs.synapse.org/rest/DELETE/thread/threadId.html
         """
-        return self.syn.restDELETE(f"/thread/{threadid}")
+        self.syn.restDELETE(f"/thread/{threadid}")
 
-    def restore_thread(self, threadid):
+    def restore_thread(self, threadid: str):
         """Restore a deleted thread
         https://rest-docs.synapse.org/rest/PUT/thread/threadId/restore.html
         """
-        return self.syn.restPUT(f"/thread/{threadid}/restore")
+        self.syn.restPUT(f"/thread/{threadid}/restore")
 
-    def pin_thread(self, threadid):
+    def pin_thread(self, threadid: str):
         """Pin a thread
         https://rest-docs.synapse.org/rest/PUT/thread/threadId/pin.html
         """
-        return self.syn.restPUT(f"/thread/{threadid}/pin")
+        self.syn.restPUT(f"/thread/{threadid}/pin")
 
-    def unpin_thread(self, threadid):
+    def unpin_thread(self, threadid: str):
         """Unpin a thread
         https://rest-docs.synapse.org/rest/PUT/thread/threadId/unpin.html
         """
-        return self.syn.restPUT(f"/thread/{threadid}/unpin")
+        self.syn.restPUT(f"/thread/{threadid}/unpin")
 
-    def get_thread_message_url(self, messagekey):
+    def get_thread_message_url(self, messagekey: str) -> dict:
         """message URL of a thread. The message URL is the URL
         to download the file which contains the thread message.
         https://rest-docs.synapse.org/rest/GET/thread/messageUrl.html
         """
         return self.syn.restGET(f"/thread/messageUrl?messageKey={messagekey}")
 
-    def post_reply(self, threadid, message):
+    def post_reply(self, threadid: str, message: str) -> Reply:
         """Create a new thread in a forum
         https://rest-docs.synapse.org/rest/POST/reply.html
 
@@ -144,14 +154,17 @@ class DiscussionApi:
             DiscussionReplyBundle
         """
         create_reply = {'threadId': threadid, 'messageMarkdown': message}
-        return self.syn.restPOST('/reply', body=json.dumps(create_reply))
+        return Reply(**self.syn.restPOST('/reply',
+                                         body=json.dumps(create_reply)))
 
-    def get_reply(self, replyid):
-        """Get a reply"""
-        return self.syn.restGET(f'/reply/{replyid}')
+    def get_reply(self, replyid: str) -> Reply:
+        """Get a reply
+        https://rest-docs.synapse.org/rest/GET/reply/replyId.html"""
+        return Reply(**self.syn.restGET(f'/reply/{replyid}'))
 
-    def get_thread_replies(self, threadid, query_filter='EXCLUDE_DELETED',
-                           limit=20, offset=0):
+    def get_thread_replies(self, threadid: str,
+                           query_filter: str = 'EXCLUDE_DELETED',
+                           **kwargs):
         """Get N number of replies for a given thread ID
         https://rest-docs.synapse.org/rest/GET/thread/threadId/replies.html
 
@@ -163,37 +176,44 @@ class DiscussionApi:
         Yields:
             list: Forum threads replies
         """
-        replies = f'/thread/{threadid}/replies?filter={query_filter}'
-        return self.syn._GET_paginated(replies, limit=limit, offset=offset)
+        replies = self.syn._GET_paginated(
+            f'/thread/{threadid}/replies?filter={query_filter}',
+            **kwargs
+        )
+        for reply in replies:
+            yield Reply(**reply)
 
-    def get_reply_message_url(self, messagekey):
+    def get_reply_message_url(self, messagekey: str) -> dict:
         """message URL of a thread. The message URL is the URL
         to download the file which contains the thread message.
         https://rest-docs.synapse.org/rest/GET/reply/messageUrl.html
         """
         return self.syn.restGET(f"/reply/messageUrl?messageKey={messagekey}")
 
-    def get_forum_threadcount(self, forumid, query_filter='EXCLUDE_DELETED'):
+    def get_forum_threadcount(self, forumid: str,
+                              query_filter: str = 'EXCLUDE_DELETED') -> int:
         """Total number of threads given forum ID
         https://rest-docs.synapse.org/rest/GET/forum/forumId/threadcount.html
         """
         threadcount = f'/forum/{forumid}/threadcount?filter={query_filter}'
-        return self.syn.restGET(threadcount)
+        return self.syn.restGET(threadcount)['count']
 
-    def get_thread_replycount(self, threadid, query_filter='EXCLUDE_DELETED'):
+    def get_thread_replycount(self, threadid: str,
+                              query_filter: str = 'EXCLUDE_DELETED') -> int:
         """Total number of replies given thread ID
         https://rest-docs.synapse.org/rest/GET/thread/threadId/replycount.html
         """
         replycount = f'/thread/{threadid}/replycount?filter={query_filter}'
-        return self.syn.restGET(replycount)
+        return self.syn.restGET(replycount)['count']
 
-    def get_forum_moderators(self, forumid):
+    def get_forum_moderators(self, forumid: str) -> Iterator[int]:
         """Get moderators given a forum ID
         https://rest-docs.synapse.org/rest/GET/forum/forumId/moderators.html
         """
         return self.syn._GET_paginated(f'/forum/{forumid}/moderators')
 
-    def get_threadcount_referencing_entities(self, entityid_list):
+    def get_threadcount_referencing_entities(self,
+                                             entityid_list: list) -> list:
         """Get list of entity and count pairs, with count is the number of
         threads that belongs to projects user can view and references
         the given entity.
@@ -204,53 +224,55 @@ class DiscussionApi:
                                  body=json.dumps(entities))
 
 
-def get_forum_threads(syn, ent, query_filter='EXCLUDE_DELETED',
-                      limit=20, offset=0):
+def get_forum_threads(syn: Synapse, ent: Union[Project, str],
+                      **kwargs) -> Iterator[Thread]:
     """
     Gets threads from a forum
 
     Args:
         syn: synapse object
         ent: Synapse Project entity or id
-        query_filter:  filter forum threads returned. Can be NO_FILTER,
-                       DELETED_ONLY, EXCLUDE_DELETED.
-                       Defaults to EXCLUDE_DELETED.
+        **kwargs: query_filter - filter forum threads returned. Can be,
+                  NO_FILTER, DELETED_ONLY, EXCLUDE_DELETED.
+                  Defaults to EXCLUDE_DELETED.
+                  limit - Number of query results
+                  offset -  Page of query result
 
     Yields:
-        list: Forum threads
+        synapseservices.Thread
+
     """
     api = DiscussionApi(syn)
     synid = id_of(ent)
     forum_obj = api.get_project_forum(synid)
-    response = api.get_forum_threads(forum_obj.id,
-                                     query_filter=query_filter,
-                                     limit=limit, offset=offset)
-    return response
+    threads = api.get_forum_threads(forum_obj.id, **kwargs)
+    return threads
 
 
-def get_thread_replies(syn, thread, query_filter='EXCLUDE_DELETED',
-                       limit=20, offset=0):
+def get_thread_replies(syn: Synapse, thread: Thread,
+                       **kwargs) -> Iterator[Reply]:
     """Gets replies of a thread
 
     Args:
         syn: synapse object
         thread: Synapse thread or id
-        query_filter:  filter forum threads returned. Can be NO_FILTER,
-                       DELETED_ONLY, EXCLUDE_DELETED.
-                       Defaults to EXCLUDE_DELETED.
+        **kwargs: query_filter: filter forum threads returned. Can be,
+                  NO_FILTER, DELETED_ONLY, EXCLUDE_DELETED.
+                  Defaults to EXCLUDE_DELETED.
+                  limit - Number of query results
+                  offset -  Page of query result
 
     Yields:
-        list: Thread replies
+        synapseservices.Reply
+
     """
     api = DiscussionApi(syn)
     threadid = id_of(thread)
-    response = api.get_thread_replies(threadid,
-                                      query_filter=query_filter,
-                                      limit=limit, offset=offset)
-    return response
+    replies = api.get_thread_replies(threadid, **kwargs)
+    return replies
 
 
-def _get_text(url):
+def _get_text(url: str):
     '''
     Get the text from a message url
 
@@ -264,27 +286,29 @@ def _get_text(url):
     return response
 
 
-def get_thread_text(syn, messagekey):
+def get_thread_text(syn: Synapse, thread: Union[Thread, str]) -> str:
     '''
-    Get thread text by the messageKey that is returned by getting thread
+    Get a thread's text
 
     Args:
         syn: Synapse object
-        messagekey: Three part key from DiscussionThreadBundle.messageKey
+        thread: challengeutils.synapseservices.Thread or its id
 
     Returns:
         str: Thread text
     '''
     api = DiscussionApi(syn)
-    url = api.get_thread_message_url(messagekey)
+    if not isinstance(thread, Thread):
+        thread = api.get_thread(thread)
+    # Get the message URL with the message key
+    url = api.get_thread_message_url(thread.messagekey)
     thread_response = _get_text(url)
     return thread_response.text
 
 
-def get_thread_reply_text(syn, messagekey):
+def get_thread_reply_text(syn, reply: Reply) -> str:
     '''
-    Get thread reply text by the messageKey that is returned by
-    getting thread replies
+    Get thread reply text
 
     Args:
         syn: Synapse object
@@ -294,12 +318,15 @@ def get_thread_reply_text(syn, messagekey):
         str: Thread text
     '''
     api = DiscussionApi(syn)
-    url = api.get_reply_message_url(messagekey)
+    if not isinstance(reply, Reply):
+        reply = api.get_reply(reply)
+    url = api.get_reply_message_url(reply.messagekey)
     thread_reply_response = _get_text(url)
     return thread_reply_response.text
 
 
-def get_forum_participants(syn, ent):
+def get_forum_participants(syn: Synapse,
+                           ent: Union[Project, str]) -> List[UserProfile]:
     '''
     Get all forum participants
 
@@ -314,7 +341,7 @@ def get_forum_participants(syn, ent):
     threads = get_forum_threads(syn, synid)
     users = set()
     for thread in threads:
-        unique_users = set(thread['activeAuthors'])
+        unique_users = set(thread.active_authors)
         users.update(unique_users)
     userprofiles = [syn.getUserProfile(user) for user in users]
     return userprofiles
@@ -356,7 +383,8 @@ def create_thread_reply(syn, threadid, message):
     return replyobj
 
 
-def copy_thread(syn, thread, project):
+def copy_thread(syn: Synapse, thread: Thread,
+                project: Union[Project, str]) -> Thread:
     """Copies a discussion thread and its replies to a project
 
     Args:
@@ -368,12 +396,13 @@ def copy_thread(syn, thread, project):
         dict: Thread bundle
     """
     new_thread_obj = _copy_thread(syn, thread, project)
-    thread_replies = get_thread_replies(syn, thread['id'])
+    thread_replies = get_thread_replies(syn, thread.id)
     for reply in thread_replies:
-        copy_reply(syn, reply, new_thread_obj['id'])
+        copy_reply(syn, reply, new_thread_obj.id)
+    return new_thread_obj
 
 
-def _copy_thread(syn, thread, project):
+def _copy_thread(syn, thread: Thread, project: Union[Project, str]) -> Thread:
     """Copies a discussion thread to a project
 
     Args:
@@ -382,14 +411,14 @@ def _copy_thread(syn, thread, project):
         project: Synapse Project or its id to copy thread to
 
     Returns:
-        dict: Thread bundle
+        synapseservices.Thread
     """
     projectid = id_of(project)
-    title = thread['title']
-    author = thread['createdBy']
+    title = thread.title
+    author = thread.createdby
     username = syn.getUserProfile(author)['userName']
-    on_behalf_of = "On behalf of @{user}\n\n".format(user=username)
-    text = get_thread_text(syn, thread['messageKey'])
+    on_behalf_of = f"On behalf of @{username}\n\n"
+    text = get_thread_text(syn, thread)
     new_thread_text = on_behalf_of + text
     new_thread_obj = create_thread(syn, projectid, title, new_thread_text)
 
@@ -408,20 +437,21 @@ def copy_reply(syn, reply, thread):
         dict: Reply bundle
     """
     threadid = id_of(thread)
-    author = reply['createdBy']
+    author = reply.createdby
     username = syn.getUserProfile(author)['userName']
     on_behalf_of = "On behalf of @{user}\n\n".format(user=username)
-    text = get_thread_reply_text(syn, reply['messageKey'])
+    text = get_thread_reply_text(syn, reply)
     new_reply_text = on_behalf_of + text
     return create_thread_reply(syn, threadid, new_reply_text)
 
 
-def copy_forum(syn, project, new_project):
+def copy_forum(syn: Synapse, project: Union[Project, str],
+               new_project: Union[Project, str]):
     """Copies the discussion forum of a project to another project
 
     Args:
         syn: synapse object
-        project: Synapse Project
+        project: Synapse Project or its id
         new_project: Synapse Project to copy forum to
     """
     threads = get_forum_threads(syn, project)
